/* This is a module which is used to remote sysrq. */
#include <linux/kernel.h>
#include <linux/version.h>
#include <linux/module.h>
#include <linux/capability.h>
#include <linux/init.h>

#define KB 1024
#define MB KB * 1024
#define GB MB * 1024

int alloc_in_pages = 0;
module_param(alloc_in_pages, int, 0);
MODULE_PARM_DESC(alloc_in_pages, "Allocate in page size (4KB)");

long alloc_size = 0;
module_param(alloc_size, long, 0);
MODULE_PARM_DESC(alloc_size, "Allocation size in KB/Pages");

long alloc_step = 10;
module_param(alloc_step, long, 1);
MODULE_PARM_DESC(alloc_step, "Allocation steps in KB/Pages");

int alloc_flags = GFP_HIGHUSER;
module_param(alloc_flags, int, GFP_HIGHUSER);
MODULE_PARM_DESC(alloc_flags, "kmalloc allocation flags");

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Menny Hamburger");
MODULE_DESCRIPTION("Allocation test module");

struct alloc_struct {
	struct list_head list;
	void *p;
};

static LIST_HEAD(alloc_list);

#if 0
#define __GFP_WAIT      ((__force gfp_t)0x10u)  /* Can wait and reschedule? */
#define __GFP_HIGH      ((__force gfp_t)0x20u)  /* Should access emergency pools? */
#define __GFP_IO        ((__force gfp_t)0x40u)  /* Can start physical IO? */
#define __GFP_FS        ((__force gfp_t)0x80u)  /* Can call down to low-level FS? */
#define __GFP_COLD      ((__force gfp_t)0x100u) /* Cache-cold page required */
#define __GFP_NOWARN    ((__force gfp_t)0x200u) /* Suppress page allocation failure warning */
#define __GFP_REPEAT    ((__force gfp_t)0x400u) /* Retry the allocation.  Might fail */
#define __GFP_NOFAIL    ((__force gfp_t)0x800u) /* Retry for ever.  Cannot fail */
#define __GFP_NORETRY   ((__force gfp_t)0x1000u)/* Do not retry.  Might fail */
#define __GFP_NO_GROW   ((__force gfp_t)0x2000u)/* Slab internal usage */
#define __GFP_COMP      ((__force gfp_t)0x4000u)/* Add compound page metadata */
#define __GFP_ZERO      ((__force gfp_t)0x8000u)/* Return zeroed page on success */
#define __GFP_NOMEMALLOC ((__force gfp_t)0x10000u) /* Don't use emergency reserves */
#define __GFP_HARDWALL   ((__force gfp_t)0x20000u) /* Enforce hardwall cpuset memory allocs */

#define __GFP_BITS_SHIFT 20     /* Room for 20 __GFP_FOO bits */
#define __GFP_BITS_MASK ((__force gfp_t)((1 << __GFP_BITS_SHIFT) - 1))

/* if you forget to add the bitmask here kernel will crash, period */
#define GFP_LEVEL_MASK (__GFP_WAIT|__GFP_HIGH|__GFP_IO|__GFP_FS| \
                        __GFP_COLD|__GFP_NOWARN|__GFP_REPEAT| \
                        __GFP_NOFAIL|__GFP_NORETRY|__GFP_NO_GROW|__GFP_COMP| \
                        __GFP_NOMEMALLOC|__GFP_HARDWALL)

/* This equals 0, but use constants in case they ever change */
#define GFP_NOWAIT      (GFP_ATOMIC & ~__GFP_HIGH)
/* GFP_ATOMIC means both !wait (__GFP_WAIT not set) and use emergency pool */
#define GFP_ATOMIC      (__GFP_HIGH | __GFP_NOWARN)
#define GFP_NOIO        (__GFP_WAIT)
#define GFP_NOFS        (__GFP_WAIT | __GFP_IO)
#define GFP_KERNEL      (__GFP_WAIT | __GFP_IO | __GFP_FS)
#define GFP_USER        (__GFP_WAIT | __GFP_IO | __GFP_FS | __GFP_HARDWALL)
#define GFP_HIGHUSER    (__GFP_WAIT | __GFP_IO | __GFP_FS | __GFP_HARDWALL | \
                         __GFP_HIGHMEM)
#endif

static long allocated = 0;
static int __init init(void)
{
	void *p;
	struct alloc_struct *as;
	int alloc_kb_step = (alloc_in_pages) ? (KB * 4) : KB;
	int alloc_step_bytes = alloc_step * alloc_kb_step;

	while (allocated < alloc_size) {
		as = kzalloc(sizeof(struct alloc_struct), GFP_KERNEL);
		if (!as) {
			printk("unable to allocate container struct\n");
			break;
		}

		list_add_tail(&as->list, &alloc_list);

		p = kzalloc(alloc_step_bytes, alloc_flags);
		if (!p) {
			printk("unable to allocate\n");
			break;
		}
		
		as->p = p;
	
		allocated += alloc_step;
		printk("allocation status: %ld\n", allocated * alloc_kb_step);	
	}

	return 0;
}

static void __exit fini(void)
{
	struct alloc_struct *as, *tmp;
	int alloc_kb_step = (alloc_in_pages) ? (KB * 4) : KB;

	list_for_each_entry_safe(as, tmp, &alloc_list, list) {
		if (as->p) {
			kfree(as->p);
			allocated -= alloc_step;
			printk("allocation status: %ld\n", allocated * alloc_kb_step);
		}
		list_del(&as->list);
		kfree(as);
	}
}

module_init(init);
module_exit(fini);
