/* This is a module which is used to remote sysrq. */
#include <linux/kernel.h>
#include <linux/version.h>
#include <linux/module.h>
#include <linux/capability.h>
#include <linux/init.h>
#include <linux/sysrq.h>
#include <linux/skbuff.h>
#include <linux/ip.h>
#include <linux/udp.h>
#include <net/checksum.h>
#include <linux/net.h>
#include <linux/in.h>
#include <linux/netdevice.h>
#include <net/ipv6.h>
#include <linux/netfilter.h>
#include <linux/netfilter_ipv6/ip6_tables.h>
#include <linux/workqueue.h>

#include "rsysrq.h"
#include "rsysrq_handle.h"
#include "sha1.c"
#include "rsysrq_protocol.c"

char *passwd = "";
module_param(passwd, charp, 0);
MODULE_PARM_DESC(passwd,"Ipt_SysRq password");

#define IPT_RETURN XT_RETURN

#define DEFAULT_TOLERANCE (86400/2)
int tolerance = DEFAULT_TOLERANCE;
module_param(tolerance, int, DEFAULT_TOLERANCE);
MODULE_PARM_DESC(tolerance,"Ipt_SysRq time tolerance between client and server in seconds");

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Marek Zelem <marek@terminus.sk>");
MODULE_DESCRIPTION("ip6tables SYSRQ module");

static int
checkentry(const char *tablename,
	   const void *entry,
	   const struct xt_target *target,
	   void *targinfo,
	   unsigned int targinfosize,
	   unsigned int hook_mask)
{
	return 1;
}

static struct ip6t_target ip6t_sysrq_reg = {
    .name       = "SYSRQ",
    .target      = target,
    .checkentry = checkentry,
    .me     = THIS_MODULE,
};

static int __init init(void)
{
	int rc;

	rc = init_rsysrq_wq();
	if (rc < 0) {
		printk(KERN_WARNING "Remote SysRq: failed to create rsysrqd_thread\n");
		return -1;
	}

	if (ip6t_register_target(&ip6t_sysrq_reg)) {
		printk(KERN_WARNING "Remote SysRq: Unable to register ipt target\n");
		return -EINVAL;
	}

	return 0;
}

static void __exit fini(void)
{
	ip6t_unregister_target(&ip6t_sysrq_reg);

	kill_rsysrq();
}

module_init(init);
module_exit(fini);
